import pygame

try:
    from cjson import dump, loads
except ImportError:
    from json import dump, loads
    
class Recorder(object):
    def __init__(self, path, filter=None, events=[], autostart=False):
        """
        """
        if filter == "only":
            self.filter = 1
        elif filter == "deny":
            self.filter = 2
        else:
            self.filter = 0
        self._events = events
        
        self.f = file(path, "w+")
        self._started = False
        
        self.tick_buffer = 0
        self.autostart = autostart
        
    def start(self):
        self.clock = pygame.time.Clock()
        self._started = True
        
    def stop(self):
        self._started = False
        
    def quit(self):
        self.f.close()
        
    def update(self, events):
        if not self._started:
            if self.autostart:
                self.start()
            else:
                return
        event_part = []
        _events = self._events
        filter = self.filter
        for event in events:
            if filter:
                if filter == 1 and event.type not in _events:
                    continue
                elif filter == 2 and event.type in _events:
                    continue
            event_part.append((event.type, event.dict))
        if event_part:
            block = (self.clock.tick()+self.tick_buffer, event_part)
            self.tick_buffer = 0
            dump(block, self.f)
            self.f.write("\n")
        else:
            self.tick_buffer += self.clock.tick()
        
        
class Player(object):
    def __init__(self, path, autostart=False):
        self.f = file(path, "r")
        self.clock = pygame.time.Clock()
        self.ticks = 0
        self._started = False
        
        self.event_block = None
        self.block_time = 0
        
        self.autostart = autostart
        
    def start(self):
        self._started = True
        self.next_block()

    def stop(self):
        self._started = False
        
    def quit(self):
        self.f.close()
        
    def next_block(self):
        line = self.f.readline().strip()
        if line:
            self.block_time, self.event_block = loads(line)
        else:
            self.stop()
                
    def get_events(self):
        if not self._started:
            if self.autostart:
                self.start()
            else:
                return []
        self.ticks += self.clock.tick()
        events = []
        while self.ticks > self.block_time:
            self.ticks -= self.block_time
            for ev_type, ev_args in self.event_block:
                events.append(pygame.event.Event(ev_type, ev_args))
                
            self.next_block()
        return events
            
    def post_events(self):
        for event in self.get_events():
            pygame.event.post(event)