import os

import pygame

class ControlDict(dict):
    defaults = dict(turn_left = pygame.K_LEFT,
                    turn_right = pygame.K_RIGHT,
                    boost = pygame.K_UP,
                    fire = pygame.K_SPACE)
    def __init__(self, filename):
        dict.__init__(self, **ControlDict.defaults)
        self.filename = filename
        self.load(filename)
        
    def load(self, filename):
        self.filename = filename
        if os.path.exists(filename):
            f = file(filename, "r")
            for line in f.readlines():
                line = line.strip()
                if line.startswith("#") or not "=" in line:
                    # comment
                    continue
                key, value = line.split("=", 1)
                try:
                    value = int(value.strip())
                except ValueError:
                    continue
                else:
                    self[key.strip()] = value
            f.close()
        else:
            print "Warning: Could not load controls from '%s'" % filename
            
    def save(self):
        f = file(self.filename, "w")
        f.write("\n".join(["=".join([k,str(v)]) for k,v in self.iteritems()]))
        f.close()
        
    def __setitem__(self, k, v):
        dict.__setitem__(self, k, v)
        self.save()

settings_path = os.path.abspath(os.path.join(os.path.dirname(__file__),"..", "controls.cfg"))
CONTROLS = ControlDict(settings_path)

CAMERA_SIZE = (400,400)

# variables
TURN_STEP = 0.01


# total number of minutes with boost on with 100% fuel
BOOST_DURATION = 1.0
# percentage points per minute
MILEAGE = 100 / BOOST_DURATION


# world settings
AIR_DRAG = 0.1


# game settings
FPS = 40
SAFE_COLOR = 128, 0, 0
ALPHA_COLOR = 255, 0, 255
BULLET_HOLE_RADIUS = 15  # < level/map dependent?

ALLOWED_LANDING_ANGLE = 45

# paths
PROJECT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),".."))
DATA_PATH = os.path.join(PROJECT_DIR, "data")

GUI_DATA_PATH = os.path.join(DATA_PATH, "gui")