import math

import pygame

from gallox.utils import load_image
from gallox.settings import *


class Bullet(pygame.sprite.Sprite):
    speed =  0.02
    image_name = "bullet"
    def __init__(self, startpos, rot, map, *groups):
        pygame.sprite.Sprite.__init__(self, *groups)
        self.map = map
        self.image, self.rect = load_image(self.image_name, -1)
        self.image = pygame.transform.rotate(self.image, rot)
        self.rect = self.image.get_rect(center=startpos)
        self.mask = pygame.mask.from_surface(self.image)
        f = self.speed * 1000. / FPS
        self.direction = (math.cos(math.radians(270-rot))* f,
                          math.sin(math.radians(270-rot))* f)
        self.exact_pos = self.rect.center
#        print "new bullet with dir", self.direction
                          
        
    def update(self, events, ticks):
        x, y = self.exact_pos
        dx, dy = self.direction
        #f = self.speed * ticks
        self.exact_pos = ex, ey = x + dx * ticks, y + dy * ticks
        self.rect.center = int(round(ex)), int(round(ey))
        self.map.check_bullet(self)
        
    def kill(self, explode=True):
        self.explode()
        pygame.sprite.Sprite.kill(self)
    
    def explode(self):
        pass
        
        
class Animation(pygame.sprite.Sprite):
    ms = 50
    width = 64
    height = 64
    tiles_x = 4
    tiles_y = 4
    tiles = 16
    name = ""
    def __init__(self, pos, *groups):
        pygame.sprite.Sprite.__init__(self, *groups)
        if type(self) == Animation:
            raise Exception, "do not use meta class directly"
        self.image = pygame.Surface((self.width,self.height))
        self.rect = self.image.get_rect(center=pos)
        self.images, r = load_image(self.name, -1)
        self.i = 0
        self.time = 0
        self.next_img()
      
    def next_img(self):
        rect = (self.i % self.tiles_y * self.width,
                self.i / self.tiles_x * self.height, self.width, self.height)
        self.image = self.images.subsurface(rect)
        self.i += 1
        if self.i >= self.tiles:
            self.kill()
    
    def update(self, events, ticks):
        self.time += ticks
        if self.time > self.ms:
            self.time -= self.ms
            self.next_img()
        return events
            
class Explosion(Animation):
    name = "explosion"
    
class SmallExplosion(Animation):
    name = "explosion_small"
    width = 32
    height = 32