import math

import pygame

from gallox.utils import load_image
from gallox.sprites.common import Bullet, Explosion
from gallox.settings import *

class Ship(pygame.sprite.Sprite):
    power = 0.001
    def __init__(self, pos, map, *groups):
        pygame.sprite.Sprite.__init__(self, *groups)
        # for debugging, delete later:
        self._start_pos = pos
        
        self.map = map
        self.image, self.rect = load_image("ship", alpha=True)
        self.rect.center = pos
        # keep copy to use for rotations
        self.orig_image = self.orig_image_normal = self.image.copy()
        self.orig_image_boost, r = load_image("ship_boost", alpha=True)
        self.mask = pygame.mask.from_surface(self.image)
        
        # yet unused
        self.munition = 100
        self.fuel = 100
        self.health = 100
        
        # indicators to replace key.set_repeat
        self.boost_on = False
        self.turn = 0
        
        # rotation and moving values
        self.rot = 0
#        self.speed = 0
        self.direction = [0, 0]
        self.plattform = None
        
    def fire(self):
        ShipBullet(self.rect.center, self.rot, self.map, self.groups())
        
    def update(self, events, ticks):
        if self.plattform:
            for event in events:
                if (event.type == pygame.KEYDOWN and 
                    event.key == CONTROLS["boost"] and self.fuel > 0):
                    self.plattform.undock()
                    self.plattform = None
            if self.plattform:
                return
        rot = self.rot
        old = self.rect.copy()
        # yet another flag
        update = False
        step = TURN_STEP
        if pygame.key.get_mods() & (pygame.K_RCTRL | pygame.K_LCTRL):
            step *= 2
        
        for event in events:
            if event.type == pygame.KEYDOWN:
                if event.key == CONTROLS["turn_left"]:
                    self.turn = -1
                elif event.key == CONTROLS["turn_right"]:
                    self.turn = 1
                elif event.key == CONTROLS["boost"] and self.fuel > 0:
                    self.boost_on = True
                    self.orig_image = self.orig_image_boost
                    update = True
                elif event.key == CONTROLS["fire"]:
                    self.fire()
                # for debugging:
                elif event.key == pygame.K_RETURN:
                    self.rect.center = self._start_pos
            elif event.type == pygame.KEYUP:
                if event.key == CONTROLS["boost"]:
                    self.boost_on = False
                    self.orig_image = self.orig_image_normal
                    update = True
                elif event.key == CONTROLS["turn_left"] and self.turn == -1:
                    self.turn = 0
                elif event.key == CONTROLS["turn_right"] and self.turn == 1:
                    self.turn = 0
                    
                
        if self.turn:
            self.rot += (- self.turn * step) * 1000. / FPS * ticks
        if self.boost_on:
            f = self.power * 1000. / FPS * ticks
            self.direction[0] += math.cos(math.radians(270-rot)) * f
            self.direction[1] += math.sin(math.radians(270-rot)) * f
            self.fuel -= MILEAGE * (ticks/ (1000. * 60.))
            if self.fuel <= 0:
                self.boost_on = False
                self.orig_image = self.orig_image_normal
                update = True
                self.fuel = 0
        # needs update after rotation or sth else (eg. boost on/off)
        if rot != self.rot or update:
            self.rot %= 360
            img = self.image = pygame.transform.rotate(self.orig_image, self.rot)
            self.rect = self.image.get_rect(center=self.rect.center)
            #! remove since normal image and boost image have the same mask
            if self.boost_on:
                img = pygame.transform.rotate(self.orig_image_normal, self.rot)
                rect = img.get_rect(center=self.rect.center)
            # always use normal image for mask
            self.mask = pygame.mask.from_surface(img)
            
            #! ???
            if self.map.collides(self):
                self.rot = rot 
                self.image = pygame.transform.rotate(self.orig_image, self.rot)
                self.rect = self.image.get_rect(center=self.rect.center)
                self.mask = pygame.mask.from_surface(self.image)
        # movement
        
        # gravity
       # air_drag = AIR_DRAG * self.direction[1] * self.direction[1] * ticks
       # print air_drag
     #  print ticks, ((0.001*1000.) / FPS ) * ticks
        self.direction[1] += ((0.0003*1000.) / FPS ) * ticks
        
        # dirty fix to remove the pause at maximum
#        if self.direction[1] < round(self.direction[1]) == 0:
#            self.direction[1] = 0
#        if int(self.direction[1] == 0):
#            print "falling: 0"
        
        self.rect.move_ip(*self.direction)
        if self.map.collides(self):
            #! remove
#            self.rect = old
#            self.direction = [0, 0]
            print "map collision"
            self.kill()
        plattform = pygame.sprite.spritecollideany(self, self.map.plattforms)
        if plattform:
            print "collide"
            if not (0 <= self.rot <= ALLOWED_LANDING_ANGLE or
                    360 - ALLOWED_LANDING_ANGLE  <= self.rot <= 360):
                print "bad landing angle:", self.rot, self.rot%360, ALLOWED_LANDING_ANGLE
                self.kill()
            if not self.rect.bottom - plattform.rect.top < 5:
                print "bad landing speed/pos"
                self.kill()
            if not plattform.rect.left < self.rect.centerx < plattform.rect.right:
                print "bad landing position"
                self.kill()
            self.image = self.orig_image_normal
            self.rect = self.image.get_rect(center=self.rect.center)
            self.mask = pygame.mask.from_surface(self.image)
            
            self.rect.bottom = plattform.rect.top - 1
            self.direction = [0, 0]
            self.rot = 0
            self.turn = 0
            self.boost_on = False
            plattform.dock(self)
            self.plattform = plattform
            
    def kill(self):
        print "killed"
        self.rot = 0
        self.turn = 0
        self.boost_on = False
        self.plattform = None
        self.rect.center = self._start_pos
        self.direction = [0,0]
    #print "killed"
        
#            print "b00m!!!  [reset]"
        
            
class ShipBullet(Bullet):
    def explode(self):
        Explosion(self.rect.center, self.map.allsprites)
