import sys, os

import pygame

import gui
from gallox.settings import *

MAIN_MENU_ITMES = ["new game", "settings", "highscores",
                   "editor", "about", "exit"]

CONTROL_NAMES = ["fire", "boost", "turn_right", "turn_left"] 

ABOUT_TEXT = """
gallox .oOo.
 -----
 
by Julian *Jug* Habrock
feedback at bytemuehle.de

have fun

^_^

"""
         
         

MAIN_STYLE = {
        "font": pygame.font.Font(os.path.join(GUI_DATA_PATH, "handwriting.ttf"), 25),
        "menubutton_textcolor": (150,150,150),
        "menubutton_textcolor_focused": (250,250,250),
        "menubutton_textcolor_active": (200,200,200),
        "about_text_textcolor": (153, 0, 153),
        }
        
class MenuButton(gui.Text):
    pass
        
class Menu(gui.GUI):
    def __init__(self, size):
        gui.GUI.__init__(self, size, style=MAIN_STYLE, pass_events=False)
        
        self.last_control = None
        self.waiting_for_new_control = False
        
        self.main_menu = gui.Layer(self)
        y = 100
        for item in MAIN_MENU_ITMES:
            t = MenuButton(self.main_menu, "auto", ("50%", y), item, 
                         anchor="midtop", highlight=True)
            y += 50
            
            t.listen("mouse_down", self.handle_main_menu)
        
        ##############
        ##############
        
        self.levels = gui.Layer(self, visible=False)
        #! fix
        y = 100
        for levelname in ["testlevel", "spam", "bacon", "eggs"]:
            t = MenuButton(self.levels, "auto", ("25%", y), 
                         levelname,
                         anchor="midtop", highlight=True)
            t.listen("mouse_down", self.handle_new_game)
            t.listen("focused", self.update_level_preview)
            y += 50
        b = MenuButton(self.levels, "auto", ("50%", y), "back", 
                         anchor="midtop", highlight=True)
        b.listen("mouse_down", self.handle_back)
        
        self.level_preview = gui.Layer(self.levels, ("50%", "100%"),
                                       ("50%", 0), visible=False)
        y = 50
        self.level_name = gui.Text(self.level_preview, "auto", ("50%", y),
                                    text="", template="name: %s", 
                                    anchor="midtop", 
                                    style={"textcolor":(255,255,255)})
        
        
        ##############
        ##############
        
        
        self.controls = gui.Layer(self, visible=False)
        y = 100
        for item in CONTROL_NAMES:
            t = MenuButton(self.controls, "auto", ("50%", y), 
                         pygame.key.name(CONTROLS[item]),
                         template="%s: %%s" % item, 
                         anchor="midtop", highlight=True)
            t.control_name = item
            y += 50
            
            t.listen("mouse_down", self.handle_controls)
        b = MenuButton(self.controls, "auto", ("50%", y), "back", 
                         anchor="midtop", highlight=True)
        b.listen("mouse_down", self.handle_back)
                     
        
        ##############
        ##############
                             
        
        self.about = gui.Layer(self, visible=False)
        # the button should have index 0
        b = MenuButton(self.about, "auto", ("50%", "80%"), "back", 
                         anchor="midtop", highlight=True)
        b.listen("mouse_down", self.handle_back)
        t = gui.Text(self.about, "auto", ("50%", "50%"), ABOUT_TEXT, 
                     anchor="center", align="centered", focusable=False,
                     style_class_name="about_text")
        
        
        self.active_layer = self.main_menu
        self.set_layer(self.main_menu)

        
    def set_focus(self, widget):
        if widget.is_ancestor(self.active_layer):
            gui.GUI.set_focus(self, widget)
            self.focus_index = self.active_layer.widgets.index(
                                                        self.focused_widget)
            
    def handle_events(self, events):
        # implement focus scrolling with arrow keys + selecting with return key
        not_used = []
        i = 0
        for event in events:
            if event.type == pygame.KEYDOWN:
                if self.waiting_for_new_control:
                    if event.key in CONTROLS.values() and  pygame.key.name(event.key) != self.last_control:
                        not_used.append(event)
                        print "key already in use"
                        continue
                    if event.key == pygame.K_UNKNOWN:
                        continue
                    elif event.key == pygame.K_ESCAPE:
                        self.waiting_for_new_control.text = self.last_control
                    else:
                        CONTROLS[self.waiting_for_new_control.control_name] = event.key
                        self.waiting_for_new_control.text = pygame.key.name(event.key)
                    self.waiting_for_new_control = None
                    continue
                elif event.key == pygame.K_UP:
                    self.roll_focus(-1)
                    continue
                elif event.key == pygame.K_DOWN:
                    self.roll_focus(1)
                    continue
                elif event.key == pygame.K_ESCAPE:
                    if self.main_menu.visible:
                        self.quit()
                    else:
                        self.set_layer(self.main_menu)
                    continue
                elif event.key in [pygame.K_RETURN, pygame.K_KP_ENTER]:
                    self.focused_widget._send_event("mouse_down")
            not_used.append(event)

        return not_used
    
    # main menu
    
    def handle_main_menu(self, widget, **kw):
        item = widget.text
        if item == "new game":
            self.set_layer(self.levels)
        elif item == "settings":
            self.set_layer(self.controls)
        elif item == "highscores":
            print "highscores"
        elif item == "editor":
            self._send_event("start_editor")
        elif item == "about":
            self.set_layer(self.about)
        elif item == "exit":
            self.quit()
            
    # new game
    
    def update_level_preview(self, widget, **kw):
        self.level_preview.show()
        self.level_name.text = widget.text
        
    def handle_new_game(self, widget, **kw):
        self._send_event("start_game", level_name=widget.text)
        
    # settings
    
    def handle_controls(self, widget, **kw):
        self.last_control = widget.text
        self.waiting_for_new_control = widget
        widget.text = "<???>"

    # universal <back> button
    
    def handle_back(self, widget, **kw):
        """
        Callback for "back" buttons.
        Changes view to main menu
        """
        self.set_layer(self.main_menu)

    def quit(self):
        self._send_event("exit")
        
    def roll_focus(self, direction):
        if not direction:
            return
        i = 0
        while i < len(self.active_layer.widgets) - 1:
            self.focus_index += direction
            self.focus_index %= len(self.active_layer.widgets)
            widget = self.active_layer.widgets[self.focus_index]
            if widget.focusable:
                self.set_focus(widget)
                break
            i += 1
        
    def set_layer(self, layer):
        self.active_layer.hide()
        self.active_layer = layer
        self.active_layer.show()
        self.focus_index = 0
        self.set_focus(self.active_layer.widgets[0])
        
    def blit(self, screen):
        screen.fill((0, 0, 0))
        gui.GUI.blit(self, screen)